/**
 * Copyright (c) 2008-2012 EBMWebsourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.api.configuration;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.util.PropertiesUtil;

import com.ebmwebsourcing.easycommons.properties.PropertiesException;
import com.ebmwebsourcing.easycommons.properties.PropertiesHelper;

/**
 * The CDK component specific extended configuration. 
 * <p>
 * An instance of this object
 * is build with a List of DOM elements. The DOM elements correspond to the
 * defined elements in the CDK component specific XML schema.
 * </p>
 * <p>
 * The parameter are stored in a key/value form.
 * </p>
 * 
 * TODO : Refactor this class in a more elegant way
 * @author Roland Naudin - EBM WebSourcing
 */
public class ConfigurationExtensions extends AbstractConfigurationParameters implements Map<String, String> {
    
    /**
     * The properties storing Place Holders values.
     */
    private Properties properties;

    /**
     * @param configurationElements
     * @param propertiesFilePath
     * @throws PEtALSCDKException
     */
    public ConfigurationExtensions(final List<?> configurationElements, final String propertiesFilePath)
            throws PEtALSCDKException {
        super(configurationElements);
        
        if (propertiesFilePath != null) {
            try {
                this.properties = PropertiesUtil.loadProperties(propertiesFilePath);
            } catch (final PropertiesException pe) {
                throw new PEtALSCDKException(pe);
            }
            PropertiesHelper.resolveMapWithNoException(this.rawCfgParams, this.properties);
        }
    }

    /**
     * @param configurationElements
     */
    public ConfigurationExtensions(final List<?> configurationElements) {
        super(configurationElements);
    }

    @Override
    public void clear() {
        this.rawCfgParams.clear();

    }

    @Override
    public boolean containsKey(final Object key) {
        return this.rawCfgParams.containsKey(key);
    }

    @Override
    public boolean containsValue(final Object value) {
        return this.rawCfgParams.containsValue(value);
    }

    @Override
    public Set<Entry<String, String>> entrySet() {
        return this.rawCfgParams.entrySet();
    }

    @Override
    public String get(final Object key) {
        return this.get((String) key);
    }

    @Override
    public String get(final String key) {
        return this.rawCfgParams.get(key);
    }

    @Override
    public boolean isEmpty() {
        return this.rawCfgParams.isEmpty();
    }

    @Override
    public Set<String> keySet() {
        return this.rawCfgParams.keySet();
    }

    @Override
    public String put(final String key, final String value) {
        if (this.properties != null) {
            try {
                return this.rawCfgParams.put(key, PropertiesHelper.resolveString(value, this.properties));
            } catch (PropertiesException pe) {
                // let the value with the place holders in case of error
                return this.rawCfgParams.put(key, value);
            }
        } else {
            return this.rawCfgParams.put(key, value);
        }
    }

    @Override
    public void putAll(final Map<? extends String, ? extends String> map) {
        if (this.properties != null) {
            PropertiesHelper.resolveMapWithNoException((Map<String, String>) map, this.properties);
        }
        this.rawCfgParams.putAll(map);
    }

    @Override
    public String remove(final Object key) {
        return this.rawCfgParams.remove(key);
    }

    @Override
    public int size() {
        return this.rawCfgParams.size();
    }

    @Override
    public Collection<String> values() {
        return this.rawCfgParams.values();
    }

    @Override
    public String toString() {
        return this.rawCfgParams.toString();
    }

}